var jm = jm || {};
if (typeof module !== 'undefined' && module.exports) {
    jm = require('jm-ms-core');
}

(function () {
    if (jm.apigateway) return;
    var ERR = jm.ERR;
    var logger = jm.getLogger('jm-apigateway');

    /**
     * apigateway
     * @class apigateway
     * @param {Object} [opts={}] 参数
     * @example
     * opts参数:{
     *  services: (必填, 需要转发的服务)
     * }
     * @returns {Object}
     * @example
     * 返回结果: apigateway对象
     */
    jm.apigateway = function (opts) {
        opts = opts || {};
        var router = jm.ms();
        jm.apigateway.emit('beforeAddServices', router, opts);

        /**
         * 添加服务
         * 支持多种参数格式, 例如
         * addService(name, {uri:uri})
         * addService(name, uri)
         * @function apigateway#addService
         * @param {String} name 服务名称
         * @param {Object} opts 参数
         * @example
         * opts参数:
         * 'http://localhost:20000/config'
         * 或者对象
         * {
     *  uri: 接口路径(必填)
     * }
         * @returns {Object}
         */
        router.addService = function (name, opts) {
            if (!opts) {
                logger.warn('addService failed. %s: %j', name, opts);
                return this;
            }
            name || (name = '');
            var uri = '/' + name;
            this.proxy(uri, opts, function (err, doc) {
                if (err) {
                    return logger.warn('addService failed. %s: %j\nreturn: %j\n%s', name, opts, doc || '', err.stack);
                }
                logger.info('addService ok. %s: %j', name, opts);
            });
            return this;
        };

        /**
         * 添加多个服务
         * @function apigateway#addServices
         * @param {Object} opts 参数
         * @example
         * opts参数:{
         *  : {uri: 'http://localhost:20000/config'},
         *  config: {uri: 'http://localhost:20000/config'},
         *  config1: http://localhost:20000/config',
         *  config2: 'ws://localhost:20000/config'
         * }
         * @returns {Object}
         */
        router.addServices = function (opts) {
            for (var name in opts) {
                this.addService(name, opts[name]);
            }
            return this;
        };

        if (opts.services) router.addServices(opts.services);
        jm.apigateway.emit('afterAddServices', router, opts);
        return router;
    };
    jm.enableEvent(jm.apigateway);
})();
